@@ -1,4 +1,3 @@
-
 .placedata <- list(
   nedlands=list(lon=115.82,lat=-31.99),
   curtin=list(lon=115.9240, lat=-32.0010),
@@ -7,7 +6,7 @@
   southpole=list(lon=0,lat=-90),
   casey=list(lon=110.524, lat=-66.282),
   mawson=list(lon=62.8667, lat=-67.6000),
-  madrid=list(lon=4+25/60, lat=40+17/60),
+  madrid=list(lon=-3.704, lat=40.417),
   aarhus=list(lon=10.21, lat=56.16),
   aalborg=list(lon=9.92, lat=57.05),
   newyorkcity=
@@ -32,13 +31,15 @@ place <- function(placename) {
 
 flatearth <- function(projection=c("atlas", "cylindrical"),
                       gdata, runlen, asp=NULL, ..., do.plot=TRUE){
-  if(missing(projection)) 
+  if(missing(projection))
     projection <- "atlas"
   if(missing(gdata)) {
-    gdata <- get("earth")$coords
+    # gdata <- get("earth")$coords
+    gdata <- globe::earth$coords
   }
   if(missing(runlen)) {
-    runlen <- get("earth")$runlen
+    # runlen <- get("earth")$runlen
+    runlen <- globe::earth$runlen
   }
   lonlat <- ensurelonlat(gdata)
   x <- lon <- lonlat$lon
@@ -77,6 +78,9 @@ flatearth <- function(projection=c("atlas", "cylindrical"),
     }
   }
   
+  # Exit if gdata is null
+  if(is.null(gdata))
+    return(invisible(matrix(numeric(0), ncol = 4)))
   # remove zeroes from run length vector
   runlen <- runlen[runlen!=0]
   # do not draw line between points numbered breaks[i] and breaks[i]+1
@@ -122,11 +126,15 @@ dot <- function(a, b) {
   sum(a * b)
 }
 
-orthogproj <- function(eye=place("nedlands"), top=place("northpole"), loc) {
+orthogproj <- function(eye, top, loc) {
   #orthogonal projection of each row of 'loc' as seen from 'eye'
   # compute orthogonal triple
-  eye <- ensure3d(eye)
-  top <- ensure3d(top)
+
+  ## Resolve eye and top
+  et <- .globepars(eye, top)
+  eye <- et$eye
+  top <- et$top
+
   if(is.matrix(loc)) {
     if(ncol(loc) != 3)
       stop("matrix \"loc\" should have 3 columns")
@@ -144,7 +152,7 @@ orthogproj <- function(eye=place("nedlands"), top=place("northpole"), loc) {
 }
 
 spatialpos <- function(lon,lat) {
-  # spatial position (x,y,z) 
+  # spatial position (x,y,z)  
   # (radius of earth = 1)
   # (origin = centre of earth)
   if(missing(lat)) {
@@ -175,17 +183,38 @@ spatialpos <- function(lon,lat) {
   )
 }
 
-
-ensure3d <- function(x) {
-  # Ensure 'x' is a single 3D vector
-  if(!is.vector(x) || length(x) != 3)
-    x <- as.vector(spatialpos(x))
-  if(!is.vector(x) || length(x) != 3)
-    stop("Can't convert to 3D point")
+ensure3d <- function(x, single = FALSE){
+  # Already a single 3D vector?
+  if(is.numeric(x) && length(x) == 3)
+    return(as.numeric(x))
+  nc <- ncol(x)
+  if(!is.null(nc) && nc == 3){
+    # If it is already a matrix or data.frame with 3 columns all is good
+    x <- as.matrix(x)
+  } else{
+    # Now we assume it is some long,lat format
+    x <- ensurelonlat(x)
+    x <- spatialpos(x)
+  }
+  if(single){
+    if(nrow(x)!=1)
+      stop("Can't convert to a single 3D point")
+    x <- as.vector(x)
+  }
   return(x)
 }
 
 ensurelonlat <- function(x) {
+  if(is.null(x)){
+    return(list(lon = NULL, lat = NULL))
+  }
+  if(is.numeric(x) && length(x) == 3){
+    x <- matrix(x, ncol = 3)
+  }
+  if(is.data.frame(x) && ncol(x) == 3){
+    x <- as.matrix(x)
+    row.names(x) <- NULL
+  }
   if(is.list(x)) {
     if(!is.null(x$lon) && !is.null(x$lat)) {
       lon <- x$lon
@@ -200,27 +229,30 @@ ensurelonlat <- function(x) {
   } else if(length(x) == 2) {
     lon <- x[1]
     lat <- x[2]
+  } else if(is.matrix(x) && ncol(x) == 3){
+    lon <- atan2(x[,2], x[,1])*180/pi
+    lat <- atan2(x[,3], sqrt(x[,1]^2+x[,2]^2))*180/pi
   } else
   stop("Don't know how to extract longitude and latitude from these data")
   return(list(lon=lon, lat=lat))
 }
 
 globeearth <- function(gdata, runlen,
-                       eye=place("nedlands"), top=place("northpole"),
+                       eye, top,
                        ..., do.plot=TRUE) {
 
   if(missing(gdata)) {
-    gdata <- get("earth")$coords
+    # gdata <- get("earth")$coords
+    gdata <- globe::earth$coords
   }
   if(missing(runlen)) {
-    runlen <- get("earth")$runlen
+    # runlen <- get("earth")$runlen
+    runlen <- globe::earth$runlen
   }
-  
-  eye <- ensure3d(eye)
-  top <- ensure3d(top)
-  
-  spos <- spatialpos(gdata[,1],gdata[,2])
-  mpos <- orthogproj(eye, top, spos)
+
+
+  spos <- spatialpos(gdata[,1], gdata[,2])
+  mpos <- orthogproj(eye = eye, top = top, spos)
 
   if(do.plot)
     plot(c(-1,1), c(-1,1), type = "n", asp=1, axes=FALSE, xlab="", ylab="")
@@ -242,22 +274,17 @@ globeearth <- function(gdata, runlen,
     ## draw globe
     a <- seq(0,2 * pi, length=360)
     lines(cos(a),sin(a),lty=2)
-  } 
+  }
   result <- cbind(x[s], y[s], x[s+1], y[s+1])
   attr(result, "piece") <- as.integer(factor(cumsum(!ok)[ok]))
   return(invisible(result))
 }
 
-globepoints <- function(loc, eye=place("nedlands"), top=place("northpole"), ...,
-                        do.plot=TRUE) {
-  
-  eye <- ensure3d(eye)
-  top <- ensure3d(top)
+globepoints <- function(loc, eye, top, ..., do.plot=TRUE) {
 
-  lonlat <- ensurelonlat(loc)
-  spos <- spatialpos(lonlat$lon,lonlat$lat)
-  
-  mpos <- orthogproj(eye, top, spos)
+  spos <- ensure3d(loc, single = FALSE)
+
+  mpos <- orthogproj(eye = eye, top = top, spos)
   x <- mpos[,1]
   y <- mpos[,2]
   ok <- (mpos[,3] < 0)
@@ -268,16 +295,12 @@ globepoints <- function(loc, eye=place("nedlands"), top=place("northpole"), ...,
   return(invisible(result))
 }
 
-globelines <- function(loc, eye=place("nedlands"), top=place("northpole"), ...,
+globelines <- function(loc, eye, top, ...,
                        do.plot=TRUE) {
-  
-  eye <- ensure3d(eye)
-  top <- ensure3d(top)
 
-  lonlat <- ensurelonlat(loc)
-  spos <- spatialpos(lonlat$lon,lonlat$lat)
-  
-  mpos <- orthogproj(eye, top, spos)
+  spos <- ensure3d(loc, single = FALSE)
+
+  mpos <- orthogproj(eye = eye, top = top, spos)
   x <- mpos[,1]
   y <- mpos[,2]
   ok <- complete.cases(mpos) & (mpos[,3] < 0)
@@ -294,14 +317,12 @@ globelines <- function(loc, eye=place("nedlands"), top=place("northpole"), ...,
   return(invisible(result))
 }
 
-globedrawlong <- function(lon, eye=place("nedlands"),
-                          top=place("northpole"), ...) {
+globedrawlong <- function(lon, eye, top, ...) {
   globelines(expand.grid(lat=c(seq(-90,90,by=1), NA), lon=lon)[,2:1],
              eye, top, ...)
 }
 
-globedrawlat <- function(lat, eye=place("nedlands"),
-                         top=place("northpole"), ...) {
+globedrawlat <- function(lat, eye, top, ...) {
   globelines(expand.grid(lon=c(seq(-180,180,by=1), NA), lat=lat), eye, top, ...)
 }
 
@@ -318,12 +339,10 @@ runifsphere.wrong <- function(n) {
   return(data.frame(lon=lon,lat=lat))
 }
 
-globearrows <- function(loc, eye=place("nedlands"), top=place("northpole"), len=0.3, ..., do.plot=TRUE) {
-  eye <- ensure3d(eye)
-  top <- ensure3d(top)
+globearrows <- function(loc, eye, top, len=0.3, ..., do.plot=TRUE) {
   spos <- spatialpos(loc[,1], loc[,2])
-  tailpos <- orthogproj(eye, top, spos)
-  headpos <- orthogproj(eye, top, (1 + len) * spos)
+  tailpos <- orthogproj(eye = eye, top = top, spos)
+  headpos <- orthogproj(eye = eye, top = top, (1 + len) * spos)
   ok <- (tailpos[,3] < 0)
   if(do.plot)
     segments(tailpos[ok,1],tailpos[ok,2],headpos[ok,1],headpos[ok,2], ...)
@@ -332,3 +351,19 @@ globearrows <- function(loc, eye=place("nedlands"), top=place("northpole"), len=
   return(invisible(result))
 }
 
+.globe.package.env <- new.env()
+
+.globepars <- function(eye, top){
+  e <- .globe.package.env
+  if(!missing(eye) && !is.null(eye)){
+    eye <- ensure3d(eye, single = TRUE)
+    assign("eye", eye, envir = e)
+  }
+  if(!missing(top) && !is.null(top)){
+    top <- ensure3d(top, single = TRUE)
+    assign("top", top, envir = e)
+  }
+  return(list(eye = get("eye", envir = e), top = get("top", envir = e)))
+}
+
+.globepars(eye = list(lon = 0, lat = 0), top = list(lon = 0, lat = 90))
